var width = 150;
var height = 150;
var countX = 3;
var countY = 7;
var connectDiff = 20;
var items = countX*countY;

var pos = new Array();
var zbuffer = new Array();
var selectedItemsId = null;
var selectedItems = null;
var lastMouseX, lastMouseY;
var timer;
var gameStart = 0;
var completed = 0;
var dir = "data/1";

function init(w, h, x, y, id) {
    width = w;
    height = h;
    countX = x;
    countY = y;
    dir = "data/"+id;

    $("#gamefield").append("<div id=\"mouse-helper\"></div>");

    for (var y = 0; y < countY; y++)
        for (var x = 0; x < countX; x++) {
            i = y * countX + x;
            //createItem(i, x*width, y*height);
            createItem(i, 20 + parseInt(Math.random() * 500), 20 + parseInt(Math.random() * 400));
        }

    zbuffer[items] = null;

    $("#mouse-helper").bind("mousedown", mouseDown);
    $("#mouse-helper").bind("mouseup", mouseUp);
    $("#mouse-helper").bind("mousemove", mouseMove);
    
    //Nutne pro Internet Explorer
    $(".item").bind("mousedown", mouseDown);
    $(".item").bind("mouseup", mouseUp);
    $(".item").bind("mousemove", mouseMove);
    
    $(window).bind("resize", updatePanelSize);
    
    initPanel();
}

function initPanel()
{
    $("#panel").css("opacity", "0.9");
    updatePanelSize();
}

function updatePanelSize()
{
    $("#buttons").css("margin-left", $(window).width()-$("#buttons").outerWidth());
    $("#panel").css("margin-left", $(window).width()-$("#panel").outerWidth());    
    $("#panel").height($(window).height());  
}

function viewTime()
{
  	var time = new Date();
	  time.setTime(time.getTime()-gameStart.getTime());
	  var seconds = time.getSeconds();
	  if(seconds < 10) seconds = "0"+seconds;
    $("#timer").html(time.getMinutes()+":"+seconds);
}

function gameOver()
{
    clearTimeout(timer);
    completed = 1;
    
    $("#gamefield").append("<h1 id=\"winner\">Blahopřejeme k úspěšnému složení puzzle!</h1>");
    $("#winner").css("margin-left", ($(window).width()-$("#winner").outerWidth()-$("#panel").outerWidth())/2);
    $("#winner").css("margin-top", ($(window).height()-$("#winner").outerHeight())/2);
    $("#winner").css("opacity", "0.9");
}

function createItem(id, x, y) {
    $("#gamefield").append("<img src=\""+dir+"/" + id + ".jpg\" width=\"" + width + "\" height=\"" + height + "\" class=\"item\" id=\"item_" + id + "\" style=\"margin-left: " + x + "px; margin-top: " + y + "px\" alt=\"\" />");

    pos[id] = $("#item_" + id).offset();
    zbuffer[id] = new Array();
    zbuffer[id][0] = id;
    $("#item_" + id).css("z-index", (items - id).toString());
}

function zbufferIndex(id) {
    return items - parseInt($("#item_" + id).css("z-index"));
}

function zbufferMoveToTop(zindex) {
    var tmp = zbuffer[zindex];

    for (var i = zindex; i > 0; i--) {
        zbuffer[i] = zbuffer[i - 1];
        for (var j = 0; j < zbuffer[i].length; j++) {
            $("#item_" + zbuffer[i][j]).css("z-index", (items - i).toString());
        }
    }

    zbuffer[0] = tmp;

    for (var i = 0; i < tmp.length; i++) {
        $("#item_" + tmp[i]).css("z-index", items.toString());
    }
}

function getSelected(x, y) {
    for (var i = 0; zbuffer[i] != null; i++) {
        for (var j = 0; j < zbuffer[i].length; j++) {
            var id = zbuffer[i][j];
            if (x >= pos[id].left && x < pos[id].left + width
            && y >= pos[id].top && y < pos[id].top + height)
                return id;
        }
    }

    return -1;
}

function mouseDown(e) {
    e.preventDefault();
    
    if(completed == 1)
    {
        return;
    }

    var id = getSelected(e.pageX, e.pageY);

    if(id == -1)
    {
        return;
    }

    var selectedZindex = zbufferIndex(id);
    selectedItemsId = zbuffer[selectedZindex];

    selectedItems = new Array();
    for (var i = 0; i < selectedItemsId.length; i++) {
        selectedItems[i] = $("#item_" + selectedItemsId[i]);
    }

    lastMouseX = e.pageX;
    lastMouseY = e.pageY;

    zbufferMoveToTop(selectedZindex);
}

function correctPosition(id) {
    var selectedId = selectedItemsId[0];
    var selectedPos = selectedItems[0].offset();
    var item = $("#item_" + id);
    var offsetX = id % countX - selectedId % countX;
    var offsetY = parseInt(id / countX) - parseInt(selectedId / countX);

    var left = selectedPos.left + offsetX * width;
    var top = selectedPos.top + offsetY * height

    item.css("margin-left", left);
    item.css("margin-top", top);
    pos[id].left = left;
    pos[id].top = top;
}

function connectItem(id) {
    var zindex = zbufferIndex(id);

    var offset = zbuffer[0].length;

    for (var i = 0; i < zbuffer[zindex].length; i++) {
        correctPosition(zbuffer[zindex][i]);
        zbuffer[0][offset + i] = zbuffer[zindex][i];
        $("#item_" + zbuffer[zindex][i]).css("z-index", items.toString());
    }

    for (var i = zindex;; i++) {
        zbuffer[i] = zbuffer[i + 1];

        if (zbuffer[i] == null) {
            break;
        }

        for (var j = 0; j < zbuffer[i].length; j++) {
            $("#item_" + zbuffer[i][j]).css("z-index", (items - i).toString());
        }
    }
}

function connectItems() {
    for (var item = 0; item < selectedItems.length; item++) {
        var current = selectedItemsId[item];
        var currentPos = selectedItems[item].offset();

        //Preskocime nejvyssi z-index - dilky v aktualni vrstve
        for (var i = 1; zbuffer[i] != null; i++) {
            //Zdvojena podminka dale je zpusbena menenim zbuffer[i] v pripade pripojeni, smula...
            for (var j = 0; zbuffer[i] != null && j < zbuffer[i].length; j++) {
                var neighbour = zbuffer[i][j]
                var neighbourPos = pos[neighbour];

                //Horni soused
                if (current >= countX && current - countX == neighbour) {
                    if (Math.abs(currentPos.top - neighbourPos.top - height) < connectDiff
                    && Math.abs(currentPos.left - neighbourPos.left) < connectDiff) {
                        connectItem(neighbour);
                    }
                    continue;
                }

                //Spodni soused
                if (current < items-countX && current + countX == neighbour) {
                    if (Math.abs(currentPos.top - neighbourPos.top + height) < connectDiff
                    && Math.abs(currentPos.left - neighbourPos.left) < connectDiff) {
                        connectItem(neighbour);
                    }
                    continue;
                }

                //Levy soused
                if (current % countX > 0 && current - 1 == neighbour) {
                    if (Math.abs(currentPos.left - neighbourPos.left - width) < connectDiff
                    && Math.abs(currentPos.top - neighbourPos.top) < connectDiff) {
                        connectItem(neighbour);
                    }
                    continue;
                }

                //Pravy soused
                if (current % countX < countX - 1 && current + 1 == neighbour) {
                    if (Math.abs(currentPos.left - neighbourPos.left + width) < connectDiff
                    && Math.abs(currentPos.top - neighbourPos.top) < connectDiff) {
                        connectItem(neighbour);
                    }
                    continue;
                }
            }
        }
    }
}

function mouseUp(e) {
    e.preventDefault();
    
    if(completed == 1)
    {
        return;
    }

    if(gameStart == 0)
    {
        gameStart = new Date();
        timer = window.setInterval("viewTime()", 1000);
    }

    if(selectedItemsId == null)
    {
        return;
    }

    for (var i = 0; i < selectedItems.length; i++) {
        pos[selectedItemsId[i]] = selectedItems[i].offset();
    }

    connectItems();

    selectedItemsId = null;
    selectedItems = null;

    if (zbuffer[0].length == items) {
        gameOver();
    }
}

function mouseMove(e) {
    e.preventDefault();
    
    if(completed == 1)
    {
        return;
    }

    if (selectedItems == null) {
        return;
    }

    for (var i = 0; i < selectedItems.length; i++) {
        selectedItems[i].css("margin-left", e.pageX - lastMouseX + pos[selectedItemsId[i]].left);
        selectedItems[i].css("margin-top", e.pageY - lastMouseY + pos[selectedItemsId[i]].top);
    }
}
